C These are routines used by display3d.for, scan3d.for, and topol.for.

C Written June 1989 by Michael W. Deem.
C Modified by Michael W. Deem 30 March 1990.
C Modified by Michael W. Deem 1 August 1990.

C This routine determines the connectivity matrix from purely 
C geometric considerations.
      logical function MK_CON(all, check)

      include 'zefsa3d.inc'

      logical all, check

      integer*2 di, i, j, k, l, list(MAX_TA)
      integer*2 i0, j0, k0, l0, l1, n
      real*4 x, y, z, x2, y2, z2, a, b, c, x_len

C clear all bonds
      di = n_symm
      if (all) di = 1
      do 10 i = 1, n_atoms*n_symm
        do 10 j = 1, MAX_CLOSE
10        conlist(1,j,i) = FREE
C loop over all atoms
      do 40 i = 1, n_atoms*n_symm, di
C loop over elements that are near this atom to find bond energy
        if (degen(i)) goto 40
        a = ta_pos(1,i)
        b = ta_pos(2,i)
        c = ta_pos(3,i)
C TO_XYZ MACRO
        x = c_x1*a + c_x2*b + c_x3*c - x_off
        y = c_y1*b + c_y2*c - y_off
        z = c_z1*c
C TO_GRD MACRO
        i0 = min(max(int((x+bond)/dx_grid)+1,1),nx_grid)
        j0 = min(max(int((y+bond)/dy_grid)+1,1),ny_grid)
        k0 = min(max(int((z+bond)/dz_grid)+1,1),nz_grid)
        l0 = max(i0-1,1)
        l1 = min(i0+1, nx_grid)
        n = 0
C z loop
        do 20 j = max(k0-1,1), min(k0+1, nz_grid)
C y loop
          do 20 k = max(j0-1,1), min(j0+1, ny_grid)
C x loop; loop over all elements under this pointer
            do 20 l = grid(1,l0,k,j), grid(1,l1,k,j)+
     |                                grid(2,l1,k,j)-1
              n = n + 1
20            list(n) = l
          i0 = 0
          do 30 j = 1, n
            l = list(j)
C create vector from atom l--the conventional origin--to atom i
            x2 = x - ta2_pos(1,l)
            y2 = y - ta2_pos(2,l)
            z2 = z - ta2_pos(3,l)
            x_len = sqrt(x2*x2 + y2*y2 + z2*z2) + 1.0E-4
            k = ta_iden(l)
            if (k .ne. i .and. .not. degen(k) .and.
     |          x_len .le. NEIGHBOR) then
              i0 = min(i0+1, MAX_CLOSE)
              conlist(1, i0, i) = k
              conlist(2, i0, i) = cell_fr(l)
            end if
30          continue
C only if requested, check for the proper number of bonds
          if (check .and. i0 .ne. n_bond(i)) goto 50
40        continue
        MK_CON = .true.
        return
50      print *,'Unique atom ',(i-1)/n_symm+1,
     |          ' has ',i0,' bonds, not ',n_bond(i), ' bonds'
        MK_CON = .false.
        return
        end      


C This routine determines the connectivity matrix by determining the
C connectivity of the unique atoms and then using symmetry
      logical function MK2_CON(check)

      include 'zefsa3d.inc'

      logical check

      logical ltmp, MK_CON
      integer*2 i, n, j, k, j0, i2, j2, imin, c0
      real*4 a, b, c, x0, y0, z0, x, y, z, rmin, len(27)

      external MK_CON, SORT_CON

      MK2_CON = .false.
      ltmp = .false.
C make unique bonds
      if (.not. MK_CON(ltmp, check)) goto 90
C loop though unique atoms
      do 80 i = 0, n_atoms*n_symm-1, n_symm
C sort bonds just found
        j = i + 1
        call SORT_CON(j)
C find a group of bonds to the same atom
        j = 1
10        n = 0
20          n = n + 1
            if (j+n .gt. MAX_CLOSE) goto 30
            if (conlist(1,j+n,i+1) .eq. conlist(1,j,i+1)) goto 20
C replicate these bonds by symmetry
30       continue
C zero bonds occur at end of bond list
         if (conlist(1,j,i+1) .eq. 0) goto 80
         do 70 i2 = 2, n_symm
            if (degen(i+i2)) goto 70
C determine what (nondegenerate) atom atom i+i2 is bonded to
            c0 = n_symm * ((conlist(1,j,i+1)-1)/n_symm)
            j0 = degn_eq(c0+mult_t(i2, conlist(1,j,i+1)-c0))
            a = ta_pos(1,i+i2)
            b = ta_pos(2,i+i2)
            c = ta_pos(3,i+i2)
C TO_XYZ MACRO
            x0 = c_x1*a + c_x2*b + c_x3*c - x_off
            y0 = c_y1*b + c_y2*c - y_off
            z0 = c_z1*c
C find distances
            do 40 k = 1, 27
              a = ta_pos(1,j0) + transl(1,k)
              b = ta_pos(2,j0) + transl(2,k)
              c = ta_pos(3,j0) + transl(3,k)
C TO_XYZ MACRO
              x = c_x1*a + c_x2*b + c_x3*c - x_off
              y = c_y1*b + c_y2*c - y_off
              z = c_z1*c
40            len(k) = (x-x0)*(x-x0) + (y-y0)*(y-y0) + (z-z0)*(z-z0)
C find the n closest atoms
            do 60 j2 = 1, n
              rmin = 1.0E38
              do 50 k = 1, 27
                if (len(k) .le. rmin) then
                  imin = k
                  rmin = len(k)
                end if
50              continue
              len(imin) = 1.0E38
C make bond
              conlist(1,j+j2-1, i+i2) = j0
60            conlist(2,j+j2-1, i+i2) = imin
70          continue
          j = j + n
          if (j .lt. MAX_CLOSE) goto 10
80      continue
      MK2_CON = .true.
90    return
      end      


C This routine sorts the bonds from atom i.
      subroutine SORT_CON(i)

      include 'zefsa3d.inc'

      integer*2 i

      integer*2 j, k, tmp

      do 10 j = 1, MAX_CLOSE-1
        do 10 k = j+1, MAX_CLOSE
          if (conlist(1,k,i) .gt. conlist(1,j,i)) then
            tmp = conlist(1,j,i)
            conlist(1,j,i) = conlist(1,k,i)
            conlist(1,k,i) = tmp
            tmp = conlist(2,j,i)
            conlist(2,j,i) = conlist(2,k,i)
            conlist(2,k,i) = tmp
          end if
10        continue
      return
      end


C This routine removes degenerate atoms from conlist array.
      logical function MK_UNI(nT)

      include 'zefsa3d.inc'

      integer*2 nT

      integer*2 i, j, k, m

      m = n_atoms*n_symm
      atommap(0) = 0
C compress array
      k=0
      do 10 i = 1, m
        if (.not. degen(i)) then
          k = k + 1
          atommap(i) = k
        end if
10      continue
      k = 0
      do 30 i = 1, m
        if (.not. degen(i)) then
          k=k+1
          n_bond(k) = n_bond(i)
          do 20 j = 1, MAX_CLOSE
            conlist(2,j,k) = conlist(2,j,i)
20          conlist(1,j,k) = atommap(conlist(1,j,i))
        end if
30      continue
      if (k .ne. nT) 
     |      print *,'N actual atoms not found on second pass'
      MK_UNI = k .eq. nT
      return
      end

 
C This routine prints out the connection matrix.
      subroutine DUMP_CON(n)
 
      include 'zefsa3d.inc'
 
      integer*2 n

      integer*2 i,j
 
      print *, '==================================================='
      do 10 i = 1, n
10      write(*,100) (conlist(1, j, i), j = 1, MAX_CLOSE)
      print *, '---------------------------------------------------'
      do 20 i = 1, n
20      write(*,100) (conlist(2, j, i), j = 1, MAX_CLOSE)
      print *, '==================================================='
      return
100   format(1X,100 I3)
      end
 

C This routine determines the number of atoms in shells 1-n centered on atom i
C A breadth-first search is employed.
C Its memory usage can be optimized to use only bits, not words, for true/false
      logical function DO_SHELL(atom, shell, n, MAX_SH, nT)

      include 'zefsa3d.inc'

      integer*2 atom, n, nT, MAX_SH, shell(MAX_SH)

      integer*2 MAX_LIST, MAX_LEN
      parameter (MAX_LIST=1000, MAX_LEN=6)
      logical*2 mark(0:MAX_ATOMS,
     |        -MAX_LEN:MAX_LEN, -MAX_LEN:MAX_LEN, -MAX_LEN:MAX_LEN),
     |          new
     |        (-MAX_LEN:MAX_LEN, -MAX_LEN:MAX_LEN, -MAX_LEN:MAX_LEN)

      common /breadth/ mark, new

      integer*2 list(4,MAX_LIST), i, j, i0, j0, k0, k1, k2, k3
      integer*2 start, stop, l, k

      DO_SHELL = .false.
C mark all cells as not visited yet
      do 10 k1 = -MAX_LEN, MAX_LEN
        do 10 k2 = -MAX_LEN, MAX_LEN
          do 10 k3 = -MAX_LEN, MAX_LEN
10          new(k1, k2, k3) = .true.
C clear original cell
      do 20 k = 1, nT
20      mark(k, 0, 0, 0) = .false.
      do 30 i = 1, n
30      shell(i) = 0
      new(0,0,0) = .false.
      list(1,1) = atom
      list(2,1) = 0
      list(3,1) = 0
      list(4,1) = 0
      mark(atom, 0, 0, 0) = .true.
      start = 1
      stop = 1
C loop over shells
      do 70 i = 1, n
        l = stop
C loop over atoms in shell i
C There will always be some atoms in the shell if we are here
        j = start-1
40        j = j + 1
          if (j .gt. MAX_LIST) j = 1
          i0 = list(1,j)
C loop over bonds of atom j
          do 60 k = 1, n_bond(i0)
            j0 = conlist(2, k, i0)
            k0 = conlist(1, k, i0)
            k1 = list(2,j) + transl(1, j0)
            k2 = list(3,j) + transl(2, j0)
            k3 = list(4,j) + transl(3, j0)
            if (max(abs(k1), abs(k2), abs(k3)) .gt. MAX_LEN) goto 100
            if (new(k1, k2, k3)) then
C clear cells not visited yet
              do 50 j0 = 1, nT
50              mark(j0, k1, k2, k3) = .false.
              new(k1, k2, k3) = .false.
            end if
C see if this atom has not been visited yet
            if (.not. mark(k0, k1, k2, k3)) then
              l = l + 1
              if (l .gt. MAX_LIST) l = 1
C make sure there is enough storage for atoms
              if (l .eq. stop) goto 90
              list(1, l) = k0
              list(2, l) = k1
              list(3, l) = k2
              list(4, l) = k3
              mark(k0, k1, k2, k3) = .true.
            end if
60          continue
          if (j .ne. stop) goto 40
C check to see no atoms were found
        if (l .eq. stop) goto 80
        start = stop+1
        if (start .gt. MAX_LIST) start = 1
        stop = l
70      shell(i) = mod(1+stop+MAX_LIST-start, MAX_LIST)
80    DO_SHELL = .true.
      return
90    print *,'Out of storage for shell atoms'
      return
100   print *,'Out of storage for cell markings'
      return
      end


C This routine sorts the passed coordination sequence array rows by
C column lexicographal order via the shell sort.
      subroutine SORT_SH(m, MAX_I, MAX_J, n_i, n_j)

      integer*2 MAX_I, MAX_J, n_i, n_j, m(MAX_I, MAX_J)

      integer*2 i, j, k, tmp

      do 40 i = 1, n_j-1
        do 40 j = i+1, n_j
          do 10 k = 1, n_i
            if (m(k,i) .lt. m(k,j)) goto 40
            if (m(k,i) .gt. m(k,j)) goto 20
10          continue
20          do 30 k = 1, n_i
              tmp = m(k,i)
              m(k,i) = m(k,j)
30            m(k,j) = tmp
40          continue
      return
      end


C This routine determines the IZA Atlas coordination sequence
      subroutine COORD(nT, action)

      include 'zefsa3d.inc'

      integer*2 nT, action

      character*80 line
      logical DO_SHELL
      integer*2 i, j, shell(MAX_COOR, MAX_ATOMS)
      integer*2 sum2, sum1

      external DO_SHELL, SORT_SH

      if ((action .ne. 1) .or. (postout .neqv. .true.)) then
        print *
        print *
        print *
      end if
      if (action .eq. 1) then
        if (postout) then
          write(line,210) 
     |   '   IZA Atlas coordination sequence         Sum'
          call PSWRITE(postunit, line)
        else
          write(*,110) 
     |   '   IZA Atlas coordination sequence         Sum'
        end if
      else
        write(*,210) 
     | '   IZA Atlas coordination sequence         Sum'
      end if
      do 10 i = 1, n_atoms
        j = (i-1)*n_symm+1
        if (.not. DO_SHELL(atommap(j), shell(1,i), MAX_COOR, MAX_COOR,
     |                     nT))
     |     goto 40
10      continue
C      call SORT_SH(shell, MAX_COOR, MAX_ATOMS, MAX_COOR, n_atoms)
      sum2=0
      do 30 i = 1, n_atoms
        sum1=0
        do 20 j=1, MAX_COOR
20        sum1=sum1+shell(j,i)
        if (action .eq. 1) then
          if (postout) then
            write(line,220) (shell(j,i), j = 1, MAX_COOR), sum1
            call PSWRITE(postunit, line)
          else
            write(*,120) (shell(j,i), j = 1, MAX_COOR), sum1
          end if
        else
          write(*,220) (shell(j,i), j = 1, MAX_COOR), sum1
        end if
        sum2=sum2+sum1
30      continue
      if (action .eq. 1) then
        if (postout) then
          write(line,230) sum2
          call PSWRITE(postunit, line)
        else
          write(*,130) sum2
        end if
      else
        write(*,230) sum2
      end if
40    return
110   format(1X, 25X, A)
120   format(1X, 25X, 10 I4, I6)
130   format(1X, 25X,38x, I8)
210   format(1X, A)
220   format(1X, 10 I4, I6)
230   format(1X, 38x, I8)
      end


C This routine determines the distance from the atom pointed to by atom i's
C   bond1 and bond2 and then adds 2.
C A breadth-first search is employed.
C Its memory usage can be optimized to use only bits, not words, for true/false
      logical function GET_DIST(atom, bond1, bond2, dist, nT)
 
      include 'zefsa3d.inc'

      integer*2 MAX_LEN
      parameter(MAX_LEN=6)
      logical*2 mark(0:MAX_ATOMS,
     |        -MAX_LEN:MAX_LEN, -MAX_LEN:MAX_LEN, -MAX_LEN:MAX_LEN),
     |          new
     |        (-MAX_LEN:MAX_LEN, -MAX_LEN:MAX_LEN, -MAX_LEN:MAX_LEN)

      common /breadth/ mark, new
 
      integer*2 atom, bond1, bond2, dist, nT
 
      integer*2 MAX_LIST
      parameter (MAX_LIST=1000)
      integer*2 list(4,MAX_LIST), i, j, i0, j0, k0, k1, k2, k3
      integer*2 start, stop, l, k, k0end, k1end, k2end, k3end
 
      GET_DIST = .false.
C mark all cells as not visited yet
      do 10 k1 = -MAX_LEN, MAX_LEN
        do 10 k2 = -MAX_LEN, MAX_LEN
          do 10 k3 = -MAX_LEN, MAX_LEN
10          new(k1, k2, k3) = .true.
C clear original cell
      do 20 k = 1, nT
20      mark(k, 0, 0, 0) = .false.
      new(0,0,0) = .false.
C save atom being searched for
      k0end = conlist(1, bond2, atom)
      k1end = transl(1, conlist(2, bond2, atom))
      k2end = transl(2, conlist(2, bond2, atom))
      k3end = transl(3, conlist(2, bond2, atom))
C get atom to start search from
      k0 = conlist(1, bond1, atom)
      k1 = transl(1, conlist(2, bond1, atom))
      k2 = transl(2, conlist(2, bond1, atom))
      k3 = transl(3, conlist(2, bond1, atom))
C clear bond1 cell
      do 30 k = 1, nT
30      mark(k, k1, k2, k3) = .false.
      new(k1, k2, k3) = .false.
C mark central atom and bond1 atom as visited
      mark(atom, 0, 0, 0) = .true.
      mark(k0, k1, k2, k3) = .true.
C start search on bond1 atom
      list(1,1) = k0
      list(2,1) = k1
      list(3,1) = k2
      list(4,1) = k3
      start = 1
      stop = 1
C loop over shells
C either the atom will be found, or memory will fill up
      i = 0
40      i = i + 1
        l = stop
C loop over atoms in shell i
C There will always be some atoms in the shell if we are here
        j = start-1
50        j = j + 1
          if (j .gt. MAX_LIST) j = 1
          i0 = list(1,j)
C loop over bonds of atom j
          do 70 k = 1, n_bond(i0)
            j0 = conlist(2, k, i0)
            k0 = conlist(1, k, i0)
C make sure this bond exists
C            if (k0 .eq. 0) goto 70
            k1 = list(2,j) + transl(1, j0)
            k2 = list(3,j) + transl(2, j0)
            k3 = list(4,j) + transl(3, j0)
            if (max(abs(k1), abs(k2), abs(k3)) .gt. MAX_LEN) goto 120
            if (new(k1, k2, k3)) then
C clear cells not visited yet
              do 60 j0 = 1, nT
60              mark(j0, k1, k2, k3) = .false.
              new(k1, k2, k3) = .false.
            end if
C see if this atom has not been visited yet
            if (.not. mark(k0, k1, k2, k3)) then
              if (k0 .eq. k0end .and. k1 .eq. k1end .and.
     |            k2 .eq. k2end .and. k3 .eq. k3end) goto 80
              l = l + 1
              if (l .gt. MAX_LIST) l = 1
C make sure there is enough storage for atoms
              if (l .eq. stop) goto 120
              list(1, l) = k0
              list(2, l) = k1
              list(3, l) = k2
              list(4, l) = k3
              mark(k0, k1, k2, k3) = .true.
            end if
70          continue
          if (j .ne. stop) goto 50
C check to see no atoms were found
        if (l .eq. stop) goto 80
        start = stop+1
        if (start .gt. MAX_LIST) start = 1
        stop = l
        goto 40
80    dist = 2 + i
      GET_DIST = .true.
      return
100    print *,'Out of storage for shell atoms'
      return
110   print *,'Out of storage for cell markings'
      return
120   print *,'Atom not found'
      return
      end
 
 
C This routine does a selection sort on an integer*2 array.
      subroutine SORT(list, n)
 
      integer*2 list(*), n
      integer*2 i, j, k

      print *,'SORT: ',n 
      do 10 i = 1, n-1
        do 10 j = i+1, n
          if (list(j) .lt. list(i)) then
            k = list(j)
            list(j) = list(i)
            list(i) = k
          end if
10        continue
      return
      end
 
 
C This routine determines the Wells circuit symbols
C It is not optimal, as it calls GET_DIST six times per atom.
C   That is, GET_DIST could be changed to search for all six distances as it
C   searches.
      subroutine CIRCUIT(nT)
 
      include 'zefsa3d.inc'
 
      integer*2 nT

      logical ok, GET_DIST
      integer*2 MAX_2LEN
      parameter (MAX_2LEN=MAX_CLOSE*(MAX_CLOSE-1)/2)
      integer*2 n, i, j, k, dist(MAX_2LEN), i1, i2, l

      external GET_DIST, SORT
 
      print *
      print *,'Circuit symbols'
      do 20 i = 1, n_atoms
        k = (i-1)*n_symm+1
        n = n_bond(atommap(k))*(n_bond(atommap(k))-1)/2
        l = 0
        ok = .true.
        do 10 i1 = 1, n_bond(atommap(k))-1
          do 10 i2 = i1+1, n_bond(atommap(k))
            l = l + 1
10          ok = ok .and. GET_DIST(atommap(k), i1, i2, dist(l), nT)
        if (ok) then
          call SORT(dist, n)
          write(*,100) i, (dist(j), j = 1, n)
        end if
20      continue
      return
100   format(1X, 'T-atom ',I2, 5X, 100 I4)
      end


C This subroutine outputs the current atom positions as a CHEM-X CSSR file.
C MK_CON or MK2_CON must be called before WR_CSSR to create conlist.
      logical function WR_CSSR(unit)

      include 'zefsa3d.inc'

      integer*2 unit

      character*4 name(MAX_UNIQ), ctmp
      logical MK2_CON, ltmp
      integer*2 i, j, i0, j0, n, m, k, l, n0, con(MAX_CLOSE),
     |          newbond(MAX_CLOSE, MAX_ATOMS), type(MAX_ATOMS),
     |          bonds(MAX_CLOSE, MAX_ATOMS)
      real*4 pos(3,MAX_ATOMS)

      external MK2_CON

      WR_CSSR = .false.

      ltmp = .false.
C make bonds, do not check that the proper number of bonds exists
      if (.not. MK2_CON(ltmp)) goto 210
C create atom names
      do 10 i = 1, n_atoms
        ctmp(1:2) = 'AT'
        ctmp(3:3) = char(48 + i/10)
        ctmp(4:4) = char(48 + mod(i,10))
10      name(i) = ctmp

      n0 = n_atoms*n_symm
      n = 0
      do 20 i = 1, n0
        do 20 j = 1, MAX_CLOSE
          bonds(j,i) = 0
20        newbond(j,i) = 0
C loop though bonds to find bonds to atoms outside unit cell
      do 50 i = 1, n0
        if (degen(i)) goto 50
        do 40 j = 1, MAX_CLOSE
          i0 = conlist(1,j,i)
          j0 = conlist(2,j,i)
          if (i0 .eq. 0) goto 40
          if (j0 .eq. 14 .or. newbond(j,i) .ne. 0) goto 40
C mark this outside atom as found
          n = n + 1
          if (n .gt. MAX_ATOMS) goto 220
          pos(1,n) = ta_pos(1,i0) + transl(1,j0)
          pos(2,n) = ta_pos(2,i0) + transl(2,j0)
          pos(3,n) = ta_pos(3,i0) + transl(3,j0)
          type(n) = (i0-1)/n_symm+1
          m = 0
          do 30 k = i, n0
            do 30 l = 1, MAX_CLOSE
              if (conlist(1,l,k) .eq. i0 .and.
     |            conlist(2,l,k).eq. j0) then
                m = m + 1
                newbond(l,k) = n+n0
                bonds(m,n) = k
              end if
30            continue
40        continue
50      continue

C write out CSSR header
      write(unit,310,ERR=210) ' ',c_a, c_b, c_c
      write(unit,320,ERR=210)
     |   c_alpha*180.0/PI, c_beta*180.0/PI, c_gamma*180.0/PI
      write(unit,330,ERR=210) n+n0, 0, 'ZEFSA3D'
      write(unit,340,ERR=210) 0,1,0,'ZEFSA3D'

C write out unit cell atoms
      do 70 i = 1, n0
        k = 1
        if (degen(i)) k = 0
        do 60 j = 1, MAX_CLOSE
          con(j) = conlist(1,j,i)
60        if (conlist(1,j,i) .ne. 0 .and. 
     |        conlist(2,j,i) .ne. 14) con(j) = newbond(j,i)
70      write(unit,300,ERR=210) 
     |      0, name((i-1)/n_symm+1), (ta_pos(j,i),j=1,3),
     |     (con(j),j=1,8), 0.0, k

C write out external atoms
      do 80 i = 1, n
80      write(unit,300,ERR=210) 0, name(type(i)), (pos(j,i),j=1,3),
     |                           (bonds(j,i),j=1,8), 0.0, 1
100     continue
      WR_CSSR = .true.
      return
210   print *,'Error writing CSSR file'
      return
220   print *,'Too many atoms outside the unit cell'
      return
300   format(I4, 1X, A4, 1X, 1X, 3(F9.5, 1X), 8I4, 1X, F7.3, 1X, I3)
310   format(22X, A6, 10X, F8.3, F8.3, F8.3)
320   format(21X, 3F8.3)
330   format(2I4, 1X, A60)
340   format(I4, I2, I1, A60)
      end
