/*

  (C) 1997 Marco Falcioni

  this routine returns the angle formed by three points a,b,c

           ^
  angle = bac = arc cos (ba . ca /|ba||ca|)

*/

#define INLINE

#include <math.h>
#include <stdlib.h>
#include "error.h"
#include "misc.h"
#include "constants.h"
#include "atoms.h"

static double oneplus = 1.0000001;
static double tiny = 1.0e-10;

INLINE
double find_angle(struct Atom * a, struct Atom * b, struct Atom * c,
		  double db, double dc)
{
  double angle = 0.0, aa = 0.0,bb = 0.0, aaa = 0.0, bbb = 0.0;
  double ba[3], ca[3];

#ifndef NOCHECKS
  if(a == 0 || b == 0 || c == 0 || dc < 0.0 || db < 0.0 ||
     a->consistent == 0 || b->consistent == 0 || c->consistent == 0) {
    push_generic_error("find_angle", "(nil) args passed (or inconsistent atoms)");
    exit_clean();
  }
#endif

  if(db < tiny || dc < tiny ) { /* degenerate case */
    push_generic_warning("find_angle", "overlapping atoms...continuing");
    return -1.0;
  }

  ba[0] = b->rx - a->rx;
  ba[1] = b->ry - a->ry;
  ba[2] = b->rz - a->rz;

  ca[0] = c->rx - a->rx;
  ca[1] = c->ry - a->ry;
  ca[2] = c->rz - a->rz;

  aaa = dot(ba,ca);
  bbb = db * dc;

  aa = aaa/bbb;

  bb = acos(aa);

  /* trap exception */
  if(isnan(bb)) {
    double cc = (aa > 0.0 ? aa : -aa);
    if(cc > oneplus) {
      push_generic_error("find_angle", "truly wrong args were fed to me");
      exit_clean();
    } else {
      return (aa > 0.0 ? 0.0 : 180.0);
    }
  }

  bb /= constPi_;

  angle = bb * 180.0;

  return angle;

}
