/*

  chain (linked list) Methods 

*/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <time.h>

#include "error.h"

#include "chain.h"
#include "cell.h"
#include "atoms.h"
#include "misc.h"
#include "link.h"


long n_chains;

struct  Chain  * new_Chain  (void) 
{ 
  struct  Chain  * me;
  me = (struct  Chain  *) Malloc(sizeof(struct  Chain ));
#ifdef DEBUG
  ++n_chains;
  printf("Chains: %ld\n", n_chains);
#endif

  me->head = 0;
  me->tail = 0;
  me->length = 0;
  return me;
} 


void delete_Chain  (struct  Chain  * This)
{ 
  if(This == 0) return;
  if(This != 0 && This->length != 0) { 
    push_generic_error ("delete_Chain", "trying to free non empty Chain");
    exit_clean();
  }
#ifdef DEBUG
  --n_chains;
  printf("Chains: %ld\n", n_chains);
#endif
  Free(This);
  return;
}  


void print_Chain  (const struct  Chain  *  Chain , FILE * stream)
{ 
  unsigned long i;
  struct  Link  * a;
  if( Chain  == 0) { 
    fprintf(stream, "Chain" " is (nil)\n");
    return;
  }	
  a =  Chain ->head;
  if(a == 0) { 
    fprintf(stream, "Chain" " is empty\n");
    return;
  }	
  fprintf(stream, "Chain" " at %p {\n", Chain );
  fprintf(stream, "Chain" "->head = %p  ", Chain ->head);
  fprintf(stream, "Chain" "->tail = %p  ", Chain ->tail);
  fprintf(stream, "Chain" "->length = %ld\n", Chain ->length);
  for(i=0;i<  Chain ->length; i++) { 
    fprintf(stream,"%ld ",i);
    if(a == 0) { 
      push_generic_error ("print_" "Chain", 
			  "end of " "Chain" " reached unexpectedly");
      print_Link  ( Chain ->tail, stream);
      return;
    } print_Link  ( a, stream);
    a = a->next;
  } fprintf(stream,"} end " "Chain" ".\n");
  return;
} 

int in_Chain  (struct  Link  * me, struct  Chain  * in_me) 
{ 
  struct  Link  * c;
  if(me == 0 || in_me == 0) { 
    push_generic_error("in_" "Chain", "(nil) args passed");
    exit_clean();
  } 
  for(c = in_me->head; c != 0; c = c->next) { 
    if(me == c) return 1;
  } 
  return 0;
} 

void add_Link  (struct  Link  * me, struct  Chain  * to_me){ 
  if(me == 0 || to_me == 0) { 
    push_generic_error("add_" "Link", "(nil) args passed");
    exit_clean();
  } 
  if(to_me->head == 0) { 
    to_me->head = me;
    me->prev = 0;
  } else { 
    to_me->tail->next = me;
    me->prev = to_me->tail;
  } 
  me->next = 0;
  to_me->tail = me;
  to_me->length++;
  return;
} 

void add_ck_Link  (struct  Link  * me, struct  Chain  * to_me) 
{ 
  struct  Link  * L;

#ifndef NOCHECKS
  if(me == 0 || to_me == 0) { 
    push_generic_error("add_" "Link" "_check", "(nil) args passed");
    exit_clean();
  } 
#endif

  if(to_me->length > 0) { 
    for (L = to_me->head; L != 0; L = L->next) { 
      if(compare_content_Link (L,me)) { 
	delete_Link (me);
	return;
      } 
    } 
  } 
  if(to_me->head == 0) { 
    to_me->head = me;
    me->prev = 0;
  } else { 
    to_me->tail->next = me;
    me->prev = to_me->tail;
  } 
  me->next = 0;
  to_me->tail = me;
  to_me->length++;
  return;
} 

struct Link * pop_Chain(struct Chain * me) 
{
  struct Link * it;
#ifndef NOCHECKS
  if(me == 0) {
    push_generic_error("pop_Chain", "(nil) args passed");
    exit_clean();
  }
#endif

  /* chain is a doubly linked list:
     there are three cases: lenght = 0, lenght = 1, length > 1 */
  if(me->length == 0) { 
    return 0; 
  } else if (me->length > 1) {   
    me->length--;
    it = me->tail;
    me->tail = it->prev;
    it->prev->next = 0;
  } else {
    it = me->head;
    me->length = 0;
    me->head = me->tail = 0;
  }
  return it;

}

void remove_Link  (struct  Link  * me, struct  Chain  * from_me)
{

#ifndef NOCHECKS 
  if(me == 0 || from_me == 0) { 
    push_generic_error("remove_" "Link", "(nil) args passed");
    exit_clean();
  } 
#endif

  if(in_Chain  (me, from_me)) { 
    if(from_me->head == me) from_me->head = me->next;
    if(from_me->tail == me) from_me->tail = me->prev;
    me->prev->next = me->next;
    if(me->next !=0)  me->next->prev = me->prev;
    delete_Link  (me);
    if(from_me->length != 0) from_me->length--;
  } else { 
    push_generic_error ("remove_" "Link",
			"trying to remove a nonexisting link");
  } 
  return;
} 

void test_Chain  (const struct  Chain  * a, FILE * stream) { 
  unsigned long i;
  struct  Link  * b;
  if(a == 0) { 
    fprintf(stream,"Null " "Chain" " \n");
    return;
  } 
  if(a->length == 0 && a->head == 0 && a->tail == 0) { 
    fprintf(stream,"Empty " "Chain" " \n");
    return;
  } 
  b = a->head;
  for(i=0; i < a->length-1; i++) { 
    b = b->next;
    if(b==0) { 
      fprintf(stream,"Length not equal to number of elements\n");
      return;
    } 
  } 
  fprintf(stream,"Healty " "Chain" " \n");
  return;
} 

struct  Chain  * cat_Chain  (struct  Chain  * a, struct  Chain  * b)
{ 
  if (a==b) return a;
  if (a == 0 || b == 0) return (a == 0 ? b : a);
  if (a->length == 0) { 
    delete_Chain  (a);
    return b;
  } else if(b->length == 0) { 
    delete_Chain  (b);
    return a;
  } a->tail->next = b->head;
  b->head->prev = a->tail;
  a->tail = b->tail;
  a->length += b->length;
  b->length = 0;
  b->head = 0;
  b->tail = 0;
  delete_Chain  (b);
  return a;
} 
void clean_Chain  (struct  Chain  * a) { 
  struct  Link  * b;
  if(a != 0 && a->length != 0) { 
    while(a->head != a->tail) { 
      b = a->tail;
      a->tail = b->prev;
      delete_Link  (b);
    } 
    delete_Link  (a->head);
  } 
  a->head = 0;
  a->tail = 0;
  a->length = 0;
  return;
} 

void sort_Chain  (struct  Chain  * me) 
{ 
  struct   Link  * item;
  if(me == 0) { 
    push_generic_error("sort" "Chain","(nil) arg passed");
    exit_clean();
  } 
  if(me->length == 0) return;
  for(item = me->head; item != 0; item = item->next) { 
    struct   Link  * next;
    for(next = item->next; next != 0; next = next->next) { 
      if(item->key > next->key) { 
	swap_Link (item, next);
      } 
    } 
  }
  return;
} 

void oogl_Chain(const struct Chain * Ch, const struct Cell * C, 
		FILE * stream) 
{
  struct Link * pp;
  
  if (Ch == 0 || C == 0 || stream == 0) {
    push_generic_error("oogl_Chain", "(nil) args passed");
    exit_clean();
  }
  
  if (Ch->length == 0) return;
  
  for (pp = Ch->head;
       pp != 0;
       pp = pp->next) {
    xyz_Atom(pp->atom, C);
    fprintf(stream, "%f %f %f\n", 
	    pp->atom->rx, pp->atom->ry, pp->atom->rz);
  }
  
  return;
}
  
