#include <stdio.h>
#include <stdlib.h>


#include "error.h"
#include "matrices.h"
#include "resources.h"
#include "stringlib.h"

#include "functions.h"
#include "symmetry.h"
#include "cell.h"
#include "grid.h"
#include "positions.h"
#include "rad_pattern.h"
#include "params.h"
#include "SP_entry.h"
#include "misc.h"
#include "link.h"
#include "oogl.h"
#include "chain.h"
#include "atoms.h"
#include "verbose.h"

int main (int argc, char ** argv)
{
  struct Cell * C = 0;
  struct Positions * T = 0;
  struct Sym_group * G = 0;
  struct Params * P = 0;
  struct Rad_pattern * RP = 0;
  struct string_list * options;

  connect_error_stream(stderr, argv[0]);

  options = append_string_to_list(0, "v");
  options = append_string_to_list(options, "m");

  args_to_resources(argc, argv, options);

  if(extract_global_resource("v")) connect_verbose_stream(stderr, argv[0]);
  
  P = read_Params(stdin);

  T = init_Positions(stdin);
  
  C = read_Cell(stdin);

  G = read_Sym_group(T->n_symmetry, stdin);
  test_Sym_group(G);

  if(C->csym == is_csym_Sym_group (G)) {
    /* no problem */
  } else {
    push_generic_error(argv[0], "Centrosymmetric group/cell mismatch: check input");
    exit_clean();
  }

  read_Positions(T, stdin);
  
  P->SP = read_SP_file(T, P->Scat_Fact_file);

  if(P->Rad_file != 0) {
#ifdef DEBUG
    int i,j;
#endif
    FILE * a;
    RP = new_Rad_pattern();
    a = fopen(P->Rad_file, "r");
    if(a == 0) { /* try to read from the data path */
      char file[200];
      strcpy(file, P->data_path); 
      strcat(file, P->Rad_file);
      a = safe_open(file, "r"); /* if this does not work we bail out */
    }
    read_Rad_pattern(RP, a);
    fclose(a);

    get_correction(RP, T, C, P->SP);

#ifdef DEBUG
    for(j = 0; j < RP->length; ++j) {
      for(i = 0; i < T->n_unique; ++i) {
	printf("spot %d atom %d %f\n", j,i, RP->xray_corr[j][i]);
      }
    }
#endif
  } else {
    push_generic_error(argv[0], "no diffraction data specified... check input");
    exit(1);
  }

  symmetrize_Positions(T,G);
  mod_Positions(T);

  merge_positions(T,P,C);
  mod_Positions(T);

  get_intensity(RP,T,C,G);
  scale_Rad_pattern(RP);
  angle_Rad_pattern(RP, C);
  sort_Rad_pattern(RP);

  if(extract_global_resource("m")) write_Rad_pattern(RP,stdout);
  else plot_Rad_pattern(RP,C,stdout); 

  fprintf(stdout, "%f\n", distance_Rad_pattern(RP));

  delete_Rad_pattern(RP);
  delete_Positions(T);
  delete_Params(P);

  return 0;
  
}

