/* 
   Created: [GJ] Fri Apr 26 05:10:35 EDT 1996
 */


#include "config.h"

#include <stdio.h>
#include <stdlib.h>

#ifdef _HAVE_STRING_H
#include <string.h>
#endif

#include <strings.h>
#include "stringlib.h"
#include "resources.h"
#include "io.h"


FILE * openfile(const char * prefix, const char * name, const char * suffix,
		const char * mode)
{
  if(prefix != 0 && name != 0 && suffix != 0) {
    FILE *fp;
    int len = strlen(prefix)+strlen(name)+strlen(suffix);
    char * full_name = new_string(len+1);
  
    if(full_name == (char *)0){
      fprintf(stderr,"%s: out of memory for a filename\n", get_app_name());
      exit(1);
    }

    /* Init full_name to empty to get rid of any garbage.
     * [Yes, this is actually necessary.]
     */
    full_name[0] = '\0';
    
    /* Construct the full name of the file. */
    full_name = strcat(full_name, prefix);
    full_name = strcat(full_name, name);
    full_name = strcat(full_name, suffix);
    
    /* Open the file. */
    fp = fopen(full_name, mode);
    if(fp == (FILE *)0){
      fprintf(stderr,"%s: could not open file %s for mode %s\n",
	      get_app_name(), full_name, mode);
    }
    
    free_string(full_name);
    return fp;
  }
  else {
    fprintf(stderr,"%s: could not open file: (nil) string parameter\n",
	    get_app_name()
	    );
    return 0;
  }
}


/* local buffer size; when this fills, the result is
 * reallocated and the process continues; this is NOT
 * some arbitrarily limited bogus buffering scheme, it is
 * completely general and correct
 */
#define StreamBuffSize_ 256

char * fgetuntil(FILE * fp, const char * terminators)
{
  char buffer[StreamBuffSize_];
  char *result = (char *)0;
  int c;
  int ibuffer = 0;
  
  while((c=getc(fp)) != EOF){
    if(in_string(c,terminators) != 0){

      /* Found a terminator. Flush the buffer
       * and bug out.
       */
      buffer[ibuffer] = '\0';
      result = append_string(result,buffer);
      ungetc(c, fp); /* put terminator back */
      return result;
    }
    
    /* Found a regular char. Add it to the buffer.
     * Check for buffer overflow and flush if necessary.
     */
    buffer[ibuffer++] = c;
    if(ibuffer >= StreamBuffSize_-1){
      buffer[ibuffer] = '\0';
      result = append_string(result,buffer);
      ibuffer = 0;
    }
  }

  /* Found EOF. Flush the buffer and bug out.
   * If the buffer was empty, then return (char *)0.
   */
  if(ibuffer == 0)
    return (char *)0;
  else {
    buffer[ibuffer] = '\0';
    result = append_string(result,buffer);

    return result;
  }
}

char * fgetuntilnot(FILE * fp, const char * ignore)
{
  char buffer[StreamBuffSize_];
  char *result = (char *)0;
  int c;
  int ibuffer = 0;
  
  while((c=getc(fp)) != EOF){
    if(in_string(c,ignore) == 0){

      /* Found something which is not in the ignore string.
       * Flush the buffer and bug out.
       */
      buffer[ibuffer] = '\0';
      result = append_string(result,buffer);
      ungetc(c, fp); /* put the grabbed non-ignore back */
      return result;
    }
    
    /* Found a regular char. Add it to the buffer.
     * Check for buffer overflow and flush if necessary.
     */
    buffer[ibuffer++] = c;
    if(ibuffer >= StreamBuffSize_-1){
      buffer[ibuffer] = '\0';
      result = append_string(result,buffer);
      ibuffer = 0;
    }
  }

  /* Found EOF. Flush the buffer and bug out.
   * If the buffer was empty, then return (char *)0.
   */
  if(ibuffer == 0)
    return (char *)0;
  else {
    buffer[ibuffer] = '\0';
    result = append_string(result,buffer);

    return result;
  }
}
#undef StreamBuffSize_


char * fgetnextline(FILE * fp)
{
  if(fp != (FILE *)0){
    char * line = fgetuntil(fp,"\n");
    getc(fp); /* eat newline */
    return line;
  }
  else {
    return 0;
  }
}


int fgetline(char * place, int size, FILE * thefile)
{
  int c = 0,i;

  for(i=0; i<size-1 && (c=getc(thefile))!=EOF && c!='\n'; ++i)
    place[i] = c;
  if(c == '\n') {
    place[i] = c;
    ++i;
  }
  place[i] = '\0';
  return i;
}



#include <ctype.h>
int emptyline(const char * linebuf, const char * commentchars)
{
  int n;
  for(n=0; linebuf[n] != '\0'; n++){
    if(isspace(linebuf[n])) {
      /* only white space, keep looking */
    }
    else {
      /* found non-white, decide what to do about it */
      if(in_string(linebuf[n], commentchars)) return 1;
      else if(! isspace(linebuf[n])) return 0;
    }
  }
  return 1;  /* purely white space */
}


/* this is a shameless arbitrary limitation on
   the size of a single field in the line buffer
 */
#define FieldBuffSize_ 512

int getcolumn(double * number, const char * linebuf, int col)    
{
  char field[FieldBuffSize_];
  const char * buf;
  int n;
  
  n = 1;
  buf = linebuf;
  /* Skip over any spaces that might sit at the beginning.
   * This is a kludge for a bug which is pretty obvious
   * when you think about it.
   */
  while(*buf==' ') buf++;
  while(sscanf(buf,"%s",field)!=EOF){
    buf += strlen(field)+1;
    if(n == col) {
      if(*field == '*'){
	return 1;
      }
      else if(sscanf(field,"%lf",number)!=1){
	return -1;
      }
    }
    while(*buf==' ') buf++;
    n++;
  }
  if(n<col)
    return -1;
  else
    return 0;
}
#undef FieldBuffSize_
