/*

  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html


*/
#include <stdio.h>
#include <stdlib.h>

#include "error.h"
#include "resources.h"
#include "stringlib.h"

#include "functions.h"
#include "positions.h"
#include "grid.h"
#include "chain.h"
#include "link.h"
#include "atoms.h"
#include "params.h"
#include "oogl.h"
#include "misc.h"
#include "conf.h"
#include "rad_pattern.h"
#include "verbose.h"

#define OOGL

int main (int argc, char ** argv)
{
  struct Params * P = 0;
  struct Cell * C = 0;
  struct Rad_pattern * RP = 0;
  struct Positions * T = 0;
  struct Sym_group * G = 0;
  struct Conf * conf = 0;
  struct Grid * Gr = 0;
  struct string_list * options = 0;

  double ene = 0.0, distance = 0.0, tot_energy = 0.0;

  connect_error_stream(stderr, argv[0]);

  options = append_string_to_list(0, "v");

  args_to_resources(argc, argv, options);

  if(extract_global_resource("v")) connect_verbose_stream(stderr, argv[0]);

  read_data(&P, &T, &C, &G, &RP, stdin);

  conf = new_Conf(T->n_unique);

  srand48(P->seed);

  Gr = new_Grid(P,C,0);

  symmetrize_Positions(T,G);
  mod_Positions(T);
  merge_positions(T,P,C);
  mod_Positions(T);

  fill_Grid(Gr, T,C);

  ene = energy(P, C, T, Gr, conf);

  if(RP) {
    get_intensity(RP, T, C);
    scale_Rad_pattern(RP);
    distance = distance_Rad_pattern(RP);
  }

  tot_energy =  P->wt_radpat * distance + ene;

  printf("%f %f %f\n", tot_energy, distance, ene);

  return 0;
  
}

