/*

  Marco Falcioni (C) 1997

  miscellaneous functions, mostly dealing with string manipulation
  also macros, REMEMBER NEVER TO EVALUATE ANYTHING IN A MACRO
  ARGUMENT!


  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html



*/

#ifndef __misc_h
#define __misc_h

/* macro dot product (faster than function call?)*/
#define dot(x,y) ((x)[0] * (y)[0] + (x)[1] * (y)[1] + (x)[2] * (y)[2])

/* 
   this is dangerous for the lack of type checking, and
   the fact that expressions in x or y are evaluated once or twice 
   depending upon their value!
*/
#define max(x,y) ((x)>(y) ? (x) : (y))
#define min(x,y) ((x)<(y) ? (x) : (y))


#define LONE_ATOM 5000.0 /* GREAT UGLINESS */

#define MAX_INTENSITY 1000.0

#ifndef Random
#define Random drand48
#define Srand srand48
#endif

#ifndef INLINE
#define INLINE
#endif

enum Rad_type {NONE, XRAY, NEUTRON, BOTH};

#ifndef MYMALLOC
#define Malloc malloc
#define Free free
#endif


#ifdef MYMALLOC
void * Malloc( size_t a);
void Free(void * p);
#endif

#ifdef DEBUG
void init_mem_test( void ); 
void tally_mem (void);
#endif

/* returns NULL if comment or empty line; strips trailing comments */
extern char * clean_line (char * line);

/* counts the number of words, excluding comments and whitespace */
extern int count_cols(const char * line);

/* reads n longs from line and stores in a new vector of longs */
extern long * read_longs(unsigned long n, const char * line);

/* gets a non commented, non whitespace line from stream */
char * get_line(char * buf, int bufsize, FILE * stream);

/* opens a file and performs routine checks, verbose errors */
FILE * safe_open(char * file, char * mode);

/* identifies machine, user, date and current directory */
void run_ID (FILE * stream);

/* gaussian deviate with unit standard deviation */
double gasdev(void);


#endif
