/*

  (C) 1999 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html

*/

#ifndef __molecule_h
#define __molecule_h

/* includes */
#include <stdio.h>

struct Atom;
struct Cell;
struct Trial;
struct Params;

struct Molecule {

  int N; /* number of atoms in molecule */
  double com[3]; /* the coordinate of the center of mass */

  struct Atom ** A; /* the list of atoms */
  double phi_M, phi_m, phi;  /* the rotation move parameters */

};


/* function prototypes */
struct Molecule * new_Molecule(int N);
struct Molecule * read_Molecule(FILE * stream);
void delete_Molecule(struct Molecule * A);
void copy_Molecule(const struct Molecule * A, struct Molecule * B);
void print_Molecule(const struct Molecule * A, FILE * stream);
void shift_Molecule(struct Molecule * A, struct Trial * T);
void write_Molecule_atoms(const int Nmol,
                       struct Molecule ** M, FILE * stream);
void com_Molecule(struct Molecule * A);
void rotate_Molecule(struct Molecule * A, double angle, double * dir);
void abc_Molecule(struct Molecule * A, const struct Cell * C);
void mod_all_Molecules(const int Nmol, 
                       struct Molecule ** M, const struct Cell * C);
void mod_Molecule(struct Molecule * A, const struct Cell * C);
void set_label_Molecule(struct Molecule * A, const char * string);
int set_phi(struct Molecule * M, struct Params * P, double fraction);

#endif
