/*
  this routine moves the T-Atoms in a random direction within a
  ball of radius passed.  We move all of the T-Atoms and then symmetrize 
  the positions.

  The new position is computed in real space. This is because we want a 
  move what wil change the energy: if we move in abc space
  we bias 


  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html



*/

#include <stdio.h>
#include <math.h>

#include "error.h"
#include "matrices.h"

#include "positions.h"
#include "misc.h"
#include "functions.h"
#include "link.h"
#include "atoms.h"
#include "chain.h"
#include "trial.h"
#include "symmetry.h"
#include "group.h"
#include "cell.h"
#include "trial.h"

void move_atom (struct Positions * T_new, 
		const unsigned int i_atom,
		const struct Trial * Delta,
		const struct Cell * C)
{
  double eps[3];
  double abc[3];
  struct Link * Ln;

#ifndef NOCHECKS
  if(T_new == 0 || Delta == 0 ||  C == 0) {
    push_generic_error("move_atom", "(nil) args passed");
    exit_clean();
  }

  if(i_atom >= T_new->n_unique) {
    push_generic_error("move_atom", "i_atom too large");
    exit_clean();
  }
#endif
  
  Ln = T_new->sym_related[i_atom]->head;
  
  /* get position of new atom in real space */

  xyz_Atom(Ln->atom, C);

  eps[0] = Ln->atom->rx + Delta->dx;
  eps[1] = Ln->atom->ry + Delta->dy;
  eps[2] = Ln->atom->rz + Delta->dz;

  /* convert real space vector eps to crystallographic coords */

  matvec_multiply_d(abc, C->xyz_to_abc, eps, 3);

  /* move atom, and mod the new position */

  move_Atom(Ln->atom, abc);
  mod_Atom(Ln->atom);

  /* move all atoms symmetrically related using Symmetry operators 
  for(N = Ln->next; N != 0; N = N->next) {
    apply_Group_el(G->list[i_group], Ln->atom, N->atom);
    mod_Atom(N->atom);
    i_group++;
  }
  */
  T_new->test = 0;
  T_new->mod = 0;
  T_new->merged = 0;
  return;
}
