/* Created: [GJ] Fri Apr 26 01:54:29 EDT 1996
 */
#ifndef GJRESOURCES_H
#define GJRESOURCES_H

struct string_list;


/* Load a key-value pair into the global repository.  */
extern void insert_global_resource(const char * key, const char * value);

/* Lookup a global resource. Returns a pointer to const string. */
extern const char * global_resource(const char * key);

/* Extract a named resource.
 * Returns a copy of the resource string, which should be
 * deleted by the client, using free_string().
 */
extern char * extract_global_resource(const char * key);

/* Extract a named resource as a double value.
 * Use given default if fails, and generate an error.
 */
extern double double_from_resource(const char * key, double def);

/* Extract a named resource as a int value.
 * Use given default if fails, and generate an error.
 */
extern int int_from_resource(const char * key, int def);

/* Extract a named resource as a bool value. (looks for "on" or "off")
 * Use given default if fails, and generate an error.
 */
extern int bool_from_resource(const char * key, int def);


/* Register a string as the main program name.         */
extern void set_app_name(const char * progname);


/* Extract the program name. Returns a pointer
 * to a static string.
 */
extern const char * get_app_name(void);


/* Load command line option keys and values into global repository.
 * Check all named options to see if they are in a given check list of
 * allowed options; sound warning if not. If checklist==0, then no
 * check is done.
 * Arguments of the form -key=val or key=val are stored as standard
 * global resource key-value pairs.
 * Arguments of the form -xxx are stored as key="xxx", val="off".
 * Arguments of the form +xxx are stored as key="xxx", val="on".
 * Arguments of the form  xxx are stored as key="xxx", val="on".
 * Return 0 on success.
 */
extern int args_to_resources(int argc, char ** argv,
			     const struct string_list * checklist
			     );


/* Scan resources into repository from a stream.
 * Return 0 on success.
 */
extern int scan_resources(FILE * fp);


/* Dump resources to a stream.
 * Use specified line prefix (useful for commenting out dumps).
 * Return 0 on success.
 */
extern int dump_resources(FILE * fp, const char * prefix);


#endif  /* GJRESOURCES_H */
