/* 
  (C) 1997 Marco Falcioni

  Splines methods. Very basic stuff...

  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "error.h"
#include "memory.h"
#include "misc.h"
#include "verbose.h"
#include "spline.h"

struct Spline * new_Spline(int n) 
{
  struct Spline * S;

  if(n <= 0) {
    push_generic_error("new_Spline", "wrong arg passed");
    exit_clean();
  }

  S = (struct Spline *) Malloc(sizeof(struct Spline));

  S->X = new_vector_d(n);
  S->Y = new_vector_d(n);
  S->Y2 = new_vector_d(n);

  S->n = n;

  return S;
}

void delete_Spline(struct Spline * S)
{
  if (S == 0) return;
  free_vector(S->X);
  free_vector(S->Y);
  free_vector(S->Y2);
  return;
}

void print_Spline(const struct Spline * S, FILE * stream) 
{
  int i;
  if(S == 0 || stream == 0) {
    push_generic_error("print_Spline","(nil) args passed");
    exit_clean();
  }
  fprintf(stream, "%d\n%f\n%f\n", S->n, S->X[0], S->X[1] - S->X[0]);
  
  for( i = 0; i < S->n; i++) {
    fprintf(stream, "%f %f\n", S->Y[i], S->Y2[i]);
  }
  return;
}

void write_Spline(const struct Spline * S, FILE * stream) 
{
  int i;
  if(S == 0 || stream == 0) {
    push_generic_error("write_Spline","(nil) args passed");
    exit_clean();
  }
  
  for( i = 0; i < S->n; i++) {
    fprintf(stream, "%f %f %f\n", S->X[i], S->Y[i], S->Y2[i]);
  }
  return;
}
/* allocates the spline */
struct Spline * read_Spline(char * file_name) 
{
  FILE * stream;
  char line[1024];
  char * p;
  int num_lines, i;
  double X0, deltaX;
  struct Spline * S;

  if(file_name == 0) {
    push_generic_error("read_spline","(nil) arg passed");
    exit_clean();
  }


  push_verbose_message("read_spline","reading spline data");

  stream = safe_open(file_name, "r");

  p = get_line(line, sizeof(line), stream);
  sscanf(p, "%d", &num_lines);

  if(num_lines < 0 ) {
    push_generic_error("read_spline","wrong number of lines");
    exit_clean();
  }

  S = new_Spline(num_lines);

  p = get_line(line, sizeof(line), stream);
  sscanf(p, "%lf", &X0);

  p = get_line(line, sizeof(line), stream);
  sscanf(p, "%lf", &deltaX);

  if(errno) {
    char buf[1024];
    sprintf(buf, "%s", strerror(errno));
    push_generic_error("read_spline",buf);
    exit_clean();
  }

  for(i = 0; i < num_lines; i ++) {
    double Y, Y2;
    p = get_line(line, sizeof(line), stream);
    sscanf(p, "%lf %lf", &Y, &Y2);
    S->X[i] = X0 + (double) i * deltaX;
    S->Y[i] = Y;
    S->Y2[i] = Y2;
  }
  if(errno) {
    char buf[1024];
    sprintf(buf, "%s", strerror(errno));
    push_generic_error("read_spline",buf);
    exit_clean();
  }

  fclose(stream);

  return S;
}

/* 
   November 23 1998
   uniformly shift the spline by a given amount: translation in the 
   Y direction with no change in the derivatives
*/

void shift_Spline(struct Spline * me, double amount)
{
  int i;
  if(me == 0) {
    push_generic_error("shift_Spline","(nil) args passed");
    exit_clean();
  }

  for(i = 0; i < me->n; i++) {
    me->Y[i] += amount;
  } 

  return;
}

