/*

  Marco Falcioni (C) 1997

  interface for symmetry operations: reading in, writing out and
  testing the multiplication table of the group.

*/

#ifndef __symmetry_h
#define __symmetry_h

#include <stdio.h>

/* from libgjm-1.3 */
struct Group;

/* data */

struct Sym_group {
  unsigned number;
  char label[10];
  char qualifier[20];
  struct Group_el ** list;
  unsigned long n_el;
  char test;
  double ** polar_matrix; /* the matrix that takes care of the 
			     polar directions if any */
  int polar; /* flag */
  int move_0; /* if the polar symmetry is maximal (P1) do not move 1 atom */
};

/* function prototypes */
struct Sym_group * read_Sym_group (unsigned long n_el, FILE * stream);
/* read the symmetry group according to the XTL convention */
struct Sym_group * read_Sym_group_XTL (unsigned long n_el, FILE * stream);

void delete_Sym_group (struct Sym_group * a);

void write_Sym_group (const struct Sym_group * a, FILE * stream);

/* print the group according to the XTL file convention */
void write_Sym_group_XTL(const struct Sym_group * a, FILE * stream);

/* testing will set the status of the test variable */
void test_Sym_group (struct Sym_group * a);

/* finds polar directions and if any constructs projector */
void find_polar_Sym_group(struct Sym_group * G);

/* is the  group centrosymmetric? */
int is_csym_Sym_group (const struct Sym_group * a);

#endif



